;============================================================================
; COMN.ASM
;
; Common module, contains only public functions.
;============================================================================




.nolist

include		..\tui.inc

.listall

include		comn.inc




.data




szTuiIni	db	MAX_PATH dup (0)
szNull		db	0,0




; Common module error handling.


; Error codes.

ERR_CM_Module	equ	1000h

ERR_Common	equ	ERR_CM_Module + 0	; (placeholder only)
ERR_Malloc	equ	ERR_CM_Module + 1
ERR_ReMalloc	equ	ERR_CM_Module + 2
ERR_DeMalloc	equ	ERR_CM_Module + 3

; Error messages.
;
; sERR_Memory has one of the following error messages appended.

sERR_Common	db	"Common module failure - ",0
sERR_Malloc	db	"Memory allocation",0
sERR_ReMalloc	db	"Memory reallocation",0
sERR_DeMalloc	db	"Memory deallocation",0
		align	dword

ErrorMsgs	dd	offset sERR_Common
		dd	offset sERR_Malloc
		dd	offset sERR_ReMalloc
		dd	offset sERR_DeMalloc

kErrors		equ	4




.code




;----------------------------------------------------------------------------
File_Exist	proc	pszFile:dword
;---------
;
; Check a file exists and return file size.
;
;
; Entry		pszFile		pointer to asciiz file name or path + name.
;
;
; Exit		carry clear	file exists, eax has file size
;
;		carry set	file not found, eax has last-error code
;----------------------------------------------------------------------------


	local	wfd		:WIN32_FIND_DATA


	invoke	FindFirstFileA, pszFile, addr wfd

	.if	(eax == INVALID_HANDLE_VALUE)

		invoke	GetLastError

		stc			; not found

	.else

		invoke	FindClose, eax

		mov	eax, wfd.nFileSizeLow

		clc			; found

	.endif


	ret


File_Exist	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
File_Open	proc	pszFile:dword,Access:dword,Mode:dword,FAttr:dword
;--------
;
; Open a file.
;
;
; Entry		pszFile		pointer to asciiz file name or path + name.
;		Access		GENERIC_READ and/or GENERIC_WRITE
;		Mode		OPEN_EXISTING or CREATE_ALWAYS
;		FAttr		file flags and attributes
;
;
; Exit		carry clear	ok, eax has file handle
;
;		carry set	failed, eax has last-error code
;----------------------------------------------------------------------------


; Open file.


	invoke	CreateFileA, pszFile, Access, NULL, NULL,
			Mode, FAttr, NULL

	.if	(eax == INVALID_HANDLE_VALUE)	; failed
		jmp	@@ExitFail
	.endif

	clc

@@Exit:

	ret

@@ExitFail:

	invoke	GetLastError
	stc
	jmp	@@Exit


File_Open	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
File_Close	proc	hFile:dword
;---------
;
; Close a file.
;
;
; Entry		hFile		file handle
;
;
; Exit		n/a
;----------------------------------------------------------------------------


	invoke	FlushFileBuffers, hFile
	invoke	CloseHandle, hFile


	ret


File_Close	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
File_ReadCl	proc	uses ebx,
			pszFile:dword,pBuf:dword,nBytes:dword,FAttr:dword
;----------
;
; Open a file, read some/all of the file to a buffer and close it.
;
; Read is from the start of the file.
; NB A buffer must be allocated and of the size required.
;
;
; Entry		pszFile		pointer to asciiz file name or path + name.
;		pBuf		pointer to buffer to read into
;		nBytes		bytes to read
;		FAttr		file flags and attributes
;
;
; Exit		carry clear	eax has bytes read
;
;		carry set	failed, eax has last-error code
;----------------------------------------------------------------------------


	local	nBytesRd	:dword		; bytes read


; Open file, checking it exists.


	invoke	CreateFileA, pszFile, GENERIC_READ, NULL, NULL,
			OPEN_EXISTING, FAttr, NULL

	.if	(eax == INVALID_HANDLE_VALUE)	; not found
		jmp	@@ExitFail
	.endif

	mov	ebx, eax


; Read file and close.


	invoke	ReadFile, ebx, pBuf, nBytes, addr nBytesRd, NULL

	push	eax
	invoke	CloseHandle, ebx
	pop	eax

	.if	(eax == 0)			; failed
		jmp	@@ExitFail
	.endif

	mov	eax, nBytesRd
	clc

@@Exit:

	ret

@@ExitFail:

	invoke	GetLastError
	stc
	jmp	@@Exit


File_ReadCl	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
File_WriteAll	proc	uses ebx,
			pszFile:dword,pBuf:dword,nBytes:dword,FAttr:dword
;------------
;
; Write data to a file, flush to disk and close it.
;
; Deletes an existing file of any attribute.
;
;
; Entry		pszFile		pointer to asciiz file name or path + name
;		pBuf		pointer to buffer to write
;		nBytes		bytes to write
;		FAttr		file flags and attributes to give new file
;
;
; Exit		carry clear	eax has bytes written
;
;		carry set	failed, eax has last-error code
;----------------------------------------------------------------------------


	local	nBytesWr	:dword		; bytes written


; Delete an existing file of the same name.
;
; This takes care of any with r/o attribute which CreateFile will not delete.


	invoke	File_Delete, pszFile


; Create file.


	invoke	CreateFileA, pszFile, GENERIC_WRITE, NULL, NULL,
			CREATE_NEW, FAttr, NULL

	.if	(eax == INVALID_HANDLE_VALUE)	; failed
		jmp	@@ExitFail
	.endif

	mov	ebx, eax


; Write file, flush to disk and close.


	invoke	WriteFile, ebx, pBuf, nBytes, addr nBytesWr, NULL

	push	eax
	invoke	FlushFileBuffers, ebx
	invoke	CloseHandle, ebx
	pop	eax

	.if	(eax == 0)			; failed
		jmp	@@ExitFail
	.endif

	mov	eax, nBytesWr
	clc

@@Exit:

	ret

@@ExitFail:

	invoke	GetLastError
	stc
	jmp	@@Exit


File_WriteAll	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
File_WriteStrg	proc	hFile:dword, pString:dword
;-------------
;
; Write asciiz string to a file.
;
; NB the file must be opened for writing.
;
;
; Entry		hFile		file handle
;
;		pString		asciiz string
;
;
; Exit		carry clear	eax has bytes written
;
;		carry set	failed, eax has last-error code
;----------------------------------------------------------------------------


	local	BytesWr		:dword		; bytes written to file


	invoke	StrLen, pString
	mov	edx, eax

	invoke	WriteFile, hFile, pString, edx, addr BytesWr, NULL

	.if	(eax == 0)			; failed
		jmp	@@ExitFail
	.endif

	mov	eax, BytesWr
	clc

@@Exit:

	ret

@@ExitFail:

	invoke	GetLastError
	stc
	jmp	@@Exit


File_WriteStrg	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
File_Delete	proc	pszFile:dword
;----------
;
; Delete a file.
;
;
; Entry		pszFile		pointer to asciiz file name or path + name.
;
;
; Exit		carry clear	file deleted ok or file did not exist
;
;		carry set	file exists, delete failed
;				eax has last-error code
;----------------------------------------------------------------------------


	local	wfd		:WIN32_FIND_DATA


	invoke	FindFirstFileA, pszFile, addr wfd

	.if	(eax == INVALID_HANDLE_VALUE)

		clc			; File not found

	.else

		invoke	FindClose, eax

		invoke	SetFileAttributesA, pszFile, FILE_ATTRIBUTE_NORMAL
		invoke	DeleteFileA, pszFile

		.if	(eax)

			clc		; done

		.else

			invoke	GetLastError

			stc		; flag error

		.endif

	.endif


	ret


File_Delete	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
File_Map	proc	uses	esi, pMMF:dword
;-------
;
; Open/create and memory map a file.
;
; NB Must be paired with File_UnMap.
;
;
; Entry		pMMF		pointer to initialised MMF structure,
;				see FILE.INC
;
;
; Exit		carry clear	success, eax has size of mapped file iff
;				loading an existing file
;
;		carry set	failed,	eax has last-error code
;
;
; Notes
;
; - The maximum file size is 2Gb.
; - On error any prior objects are released.
;----------------------------------------------------------------------------


	local	mAccess		:dword		; mapped file access flags
	local	mProtect	:dword		; mapped object protection
	local	FileSize	:dword		; size of file/object
	local	LastError	:dword


; Setup.


	mov	esi, pMMF
	mov	eax, [esi + MMF.Access]

	.if	(eax == GENERIC_READ)
		mov	mAccess, PAGE_READONLY
		mov	mProtect, FILE_MAP_READ
	.else
		mov	mAccess, PAGE_READWRITE
		mov	mProtect, FILE_MAP_WRITE
	.endif


; Open file.


	; Load returned size and detect no file if OPEN_EXISTING.

	mov	eax, [esi + MMF.Mode]

	.if	(eax == OPEN_EXISTING)

		invoke	File_Exist, [esi + MMF.pszFile]
		jc	@@ExitFail

		mov	FileSize, eax

	.endif


	invoke	CreateFileA, [esi + MMF.pszFile], [esi + MMF.Access], NULL,
			NULL, [esi + MMF.Mode], [esi + MMF.Attr], NULL

	.if	(eax == INVALID_HANDLE_VALUE)	; failed

		invoke	GetLastError
 		mov	LastError, eax
 		jmp	@@ExitFail

 	.endif

	mov	[esi + MMF.hFile], eax


; Map all of the file into memory.


	invoke	CreateFileMapping, eax, NULL, mAccess, 0,
			[esi + MMF.nFile], NULL

	.if	(eax == 0)			; failed

		invoke	GetLastError
 		mov	LastError, eax
 		jmp	@@ExitFail_1

	.endif

	mov	[esi + MMF.hMFile], eax


	invoke	MapViewOfFile, eax, mProtect, 0, 0, 0

	.if	(eax == 0)			; failed

		invoke	GetLastError
 		mov	LastError, eax
 		jmp	@@ExitFail_2

	.endif

	mov	[esi + MMF.pMFile], eax


; And Exit.


	mov	eax, FileSize

	clc


@@Exit:


	ret


@@ExitFail_2:

	invoke	CloseHandle, [esi + MMF.hMFile]	; release map handle

@@ExitFail_1:

	invoke	CloseHandle, [esi + MMF.hFile]	; release file handle

@@ExitFail:

	mov	eax, LastError
	stc
	jmp	@@Exit


File_Map	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
File_UnMap	proc	uses esi, pMMF:dword
;---------
;
; Close a memory mapped file.
;
;
; Entry		pMMF		pointer to initialised MMF structure,
;				see FILE.INC
;
;
; Exit		n/a
;----------------------------------------------------------------------------


	mov	esi, pMMF

	invoke	UnmapViewOfFile, [esi + MMF.pMFile]	; unmap memory
	invoke	CloseHandle, [esi + MMF.hMFile]		; release map handle
	invoke	CloseHandle, [esi + MMF.hFile]		; release file handle

	mov	eax, [esi + MMF.Mode]

	.if	(eax)

		; Mode is delete file.

		invoke	File_Delete, [esi + MMF.pszFile]

	.endif


	ret


File_UnMap	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
File_TuiSetIni	proc
;-------------
;
; Set Tui initialisation file path and name.
;
; This must be called before any access to the initialisation file.
;
;
; Entry		n/a
;
;
; Exit		eax		pointer to ini file path + name string
;----------------------------------------------------------------------------


	invoke	GetModuleFileName, G_hTui, addr szTuiIni, MAX_PATH

	; Replace 'exe' with 'ini' to give "tui.ini".

	invoke	StrLen, addr szTuiIni
	mov	edx, "ini"
	mov	dword ptr szTuiIni[eax - 3], edx

	mov	eax, offset szTuiIni


	ret


File_TuiSetIni	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
File_TuiQryIni	proc
;-------------
;
; Query if Tui initalisation file exists.
;
;
; Entry		n/a
;
;
; Exit		carry clear	file exists, eax has file size
;
;		carry set	file not found
;----------------------------------------------------------------------------


	invoke	File_Exist, offset szTuiIni


	ret


File_TuiQryIni	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
File_RdIniKey	proc	pszSection:dword,pszKey:dword,pszStrg:dword,Len:dword
;------------
;
; Read Key string from Tui initialisation file.
;
;
; Entry		pszSection	pointer to asciiz Section name
;		pszKey		pointer to asciiz Key name
;		pszStrg		pointer to buffer for returned key string
;		Len		length of return buffer
;
;
; Exit		eax		number of char copied to buffer, excluding
;				terminating null
;
;		szStrg		loaded with returned key string or NULL
;				string if no Key string found
;----------------------------------------------------------------------------


	invoke	GetPrivateProfileString, pszSection, pszKey, addr szNull,
			pszStrg, Len, addr szTuiIni


	ret


File_RdIniKey	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
File_WrIniKey	proc	pszSection:dword,pszKey:dword,pszStrg:dword
;------------
;
; Write Key string to Tui initialisation file.
;
;
; Entry		pszSection	pointer to asciiz Section name
;				if Section does not exist it is created
;
;		pszKey		pointer to asciiz Key name
;				if Key does not exist it is created.
;
;		pszStrg		pointer to asciiz string to write
;				if the string is NULL the key is deleted
;
;
; Exit		n/a
;----------------------------------------------------------------------------


; Set the string.


	invoke	WritePrivateProfileString, pszSection, pszKey, pszStrg,
			addr szTuiIni


	ret


File_WrIniKey	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
Mem_Alloc	proc	ReqBytes:dword
;--------
;
; Allocate a fixed memory block and zero it.
;
;
; Entry		ReqBytes	bytes required
;
;
; Exit		eax		pointer to memory block
;----------------------------------------------------------------------------


	invoke	GlobalAlloc, GPTR, ReqBytes

	.if	(eax == NULL)			; failed

		invoke	GetLastError
		mov	G_APIErrCode, eax

		invoke	RaiseException, ERR_Malloc,
					0, kErrors, addr ErrorMsgs

	.endif


	ret


Mem_Alloc	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
Mem_ReAlloc	proc	uses esi, phBlock:dword, ReqBytes:dword
;----------
;
; Re-Allocate a memory block.
;
;
; Entry		phBlock		pointer to variable containing block handle
;
;		ReqBytes	bytes required
;
;
; Exit		[hBlock], eax	new pointer to memory block
;----------------------------------------------------------------------------


	mov	esi, phBlock
	mov	eax, [esi]

	invoke	GlobalReAlloc, eax, ReqBytes, GMEM_MOVEABLE or GMEM_ZEROINIT

	.if	(eax == NULL)			; failed

		invoke	GetLastError
		mov	G_APIErrCode, eax

		invoke	RaiseException, ERR_ReMalloc,
					0, kErrors, addr ErrorMsgs

	.endif

	mov	[esi], eax


	ret


Mem_ReAlloc	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
Mem_DeAlloc	proc	uses esi, phBlock:dword
;----------
;
; De-Allocate a memory block.
;
;
; Entry		phBlock		pointer to variable containing block handle
;
;
; Exit		[hBlock], eax	set to NULL
;----------------------------------------------------------------------------


	mov	esi, phBlock
	mov	eax, [esi]

	.if	(eax)

		invoke	GlobalFree, eax

		.if	(eax)			; failed

			invoke	GetLastError
			mov	G_APIErrCode, eax

			invoke	RaiseException, ERR_DeMalloc,
						0, kErrors, addr ErrorMsgs

		.endif

	.endif

	xor	eax, eax
	mov	[esi], eax


	ret


Mem_DeAlloc	endp
;----------------------------------------------------------------------------




end		; Common module.




;============================================================================
