;============================================================================
; HTM.ASM
;
; HTML file text thumbnail handler.
;
; Extracts text between the html tag '<title>' and the next '<' delimiter.
;============================================================================




; Model ---------------------------------------------------------------------


.586p
.model		flat, stdcall		; MS Windows
option		casemap :none		; case sensitive




; Masm32 --------------------------------------------------------------------


.nolist

include		\masm32\include\windows.inc
include		\masm32\include\kernel32.inc

includelib	\masm32\lib\kernel32.lib

include		\masm32\macros\macros.asm

.listall




; Functions.

DllEntry	proto	hInstDLL:dword,reason:dword,unused:dword
PlugSetup	proto	pTuiFuncs:dword
HTMPlugin	proto	pszFile:dword,Item:dword




.data




; Plugin structures.

include		..\plugin.inc




hDll		dd	0			; handle, module

HTM		PLGSpec	<offset HTMPlugin,".HTM","HTML","html">
		align	dword

TitleTxt	db	"<title>"
nTitleTxt	equ	$-TitleTxt
		align	dword




.code




;----------------------------------------------------------------------------
DllEntry	proc	hInstDLL:dword,reason:dword,unused:dword
;-------
;
; Entry function, called by system at load.
;
;
; Exit		eax	true	load ok
;			false	load or init failed, system will abort load
;----------------------------------------------------------------------------


	.if	(reason == DLL_PROCESS_ATTACH)

		; Use Tui's handle.

		invoke	GetModuleHandle, NULL
		mov	hDll, eax

		; Perform any local initialisation here.

		mov	eax, TRUE

	.elseif	(reason == DLL_THREAD_DETACH)

		; Thread specific initialisation.

	.elseif	(reason == DLL_THREAD_ATTACH)

		; Thread specific de-initialisation.

	.elseif	(reason == DLL_PROCESS_DETACH)

		; Perform any local de-initialisation here.

	.endif


	ret


DllEntry	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
PlugSetup	proc	export	uses esi, pTuiFuncs:dword
;--------
;
; Plugin setup, called by Tui at present in DB.ASM/LoadGrpSfx.
;
;
; Entry		pTuiFuncs	pointer to Tui internal function block
;
;
; Exit		eax		pointer to plugin details structure
;----------------------------------------------------------------------------


; Required initialisation of Tui functions.


	mov	esi, pTuiFuncs
	mov	eax, [esi]
	mov	pFile_Map, eax
	mov	eax, [esi + 4]
	mov	pFile_UnMap, eax
	mov	eax, [esi + 8]
	mov	pInStrgI, eax


; This is all the setup routine needs to do for itself.


	mov	eax, offset HTM


	ret


PlugSetup	endp
;----------------------------------------------------------------------------




;----------------------------------------------------------------------------
HTMPlugin	proc	uses ebx esi edi, pszFile:dword,Item:dword
;--------
;
; HTML file plugin.
;
;
; Entry		szFile		pointer to file path + name
;		Item		Time display Item number
;
;
; Exit		eax		length of text in output buffer
;				0 if failed
;
;		edx		pointer to output buffer
;----------------------------------------------------------------------------


	local	TmpFile		:MMF
	local	FileSize	:dword


; Check object array to see if this Item's synopsis is loaded already.


	mov	eax, Item
	mov	ebx, eax
	and	ebx, nTxtObj-1		; Index,
	mov	edi, ebx
	shl	edi, nCharTTMult	; (calc ptr to object
	add	edi, offset TxtObjArr	;  in object array)
	shl	ebx, TxtObjRecMult	; * bytes per object,
	cmp	eax, TxtObj[ebx].Item	; = Object.
	jz	@@Exit


; No, mappit.


	mov	eax, pszFile
	mov	TmpFile.pszFile, eax
	mov	TmpFile.nFile, 0
	mov	TmpFile.Access, GENERIC_READ
	mov	TmpFile.Mode, OPEN_EXISTING
	mov	TmpFile.Attr, FAttrAll

	invoke	_File_Map, addr TmpFile
	mov	FileSize, eax

	.if	(CARRY?)			; failed

		xor	eax, eax
		mov	TxtObj[ebx].Len, eax
		jmp	@@Exit

	.elseif	(eax == 0)			; no data

		mov	TxtObj[ebx].Len, eax
		jmp	@@ExitUnMap

	.endif


; Scan for Title string and load to output buffer.


	mov	esi, TmpFile.pMFile
	invoke	_InStrgI, 0, esi, eax, addr TitleTxt, nTitleTxt

	.if	(!CARRY?)

		add	esi, eax		; Move to end of Title
		add	esi, nTitleTxt		;  delimiter.
		xor	ecx, ecx

		.repeat

			; Take everything until next delimiter '<'.

			mov	al, [esi + ecx]
			.break	.if (al == '<')
			mov	[edi + ecx], al
			inc	ecx

		.until	(ecx == nCharTxtThumb)

		mov	TxtObj[ebx].Len, ecx

	.else

		xor	eax, eax
		mov	TxtObj[ebx].Len, eax

	.endif


; Unload the file and exit.


@@ExitUnMap:

	mov	TmpFile.Mode, FALSE		; don't delete!
	invoke	_File_UnMap, addr TmpFile


@@Exit:

	mov	eax, TxtObj[ebx].Len
	mov	edx, edi


	ret


HTMPlugin	endp
;----------------------------------------------------------------------------




end	DllEntry




;============================================================================
