;============================================================================
; PLUGIN.INC
;
; Text Thumbnail plugin structures and data.
;
; This file must be included in a Plugin's data section.
;============================================================================




; Plugin specification
; --------------------
;
; (NB copied from tui.inc)

PLGSpec		struct
  pPLG		dd	?		; address of plugin handler
  Sfx1		db	4 dup (?)	; 4 char file suffix
  Sfx2		db	4 dup (?)	; possible additional suffix
  szName	db	16 dup (?)	; 15 char plugin name + terminator
PLGSpec		ends




; Text Thumbnail specification and data structures
; ------------------------------------------------


; Absolutes, copied from time.asm.

ItemThumb	equ	5		; number of Thumbnails, + 1
nCharTxtThumb	equ	128		; number of characters in a Thumbnail
nCharTTMult	equ	7		; 2^7 = 128


; Text object array.
;
; This provides buffering of four thumbnails to prevent constant reloading.
; (cf Text objects in time.asm)

TxtObjStruc	struc
  Item		dd	0			; Item index
  Len		dd	0			; length of actual text
TxtObjStruc	ends

TxtObjRecSize	equ	sizeof TxtObjStruc	; 8 bytes
TxtObjRecMult	equ	3			; *8

nTxtObj		equ	ItemThumb-1		; (index 0..ItemThumb-2)

; Initialise with Item = -1 to indicate no image loaded (Items are 0 up).

TxtObj		TxtObjStruc nTxtObj dup (<-1,0>)

; Text object array.

TxtObjArr	db	nCharTxtThumb*nTxtObj dup (0)




; Tui functions available for plugins
; -----------------------------------
;
; The pointer locations here must be initialised before use - see the
; PlugSetup function.
;
; The functions use their original names prefixed with '_'. They must
; be invoked as per the originals.


; File_Map	proto	pMMF:dword

pFile_Map	dd	0
szFile_Map	db	"File_Map",0
		align	dword
_File_Map	DDPROTO	(pFile_Map,1)

; File_UnMap	proto	pMMF:dword

pFile_UnMap	dd	0
szFile_UnMap	db	"File_UnMap",0
		align	dword
_File_UnMap	DDPROTO	(pFile_UnMap,1)

; InStrgI	proto	nOfs:dword,pSrc:dword,nSrc:dword,pPat:dword,nPat:dword

pInStrgI	dd	0
szInStrgI	db	"InStrgI",0
		align	dword
_InStrgI		DDPROTO	(pInStrgI,5)


; Useful definitions for File_Map/UnMap.
; (NB copied from \comn\comn.inc)


; MMF specification structure for use with File_Map and File_UnMap.

MMF		struc
  pszFile	dd	?	; pointer, asciiz file name
  nFile		dd	?	; New file:	 size in bytes
  				; Existing:	0 (NB)
  Access	dd	?	; Access mode:
  				;   GENERIC_READ and/or GENERIC_WRITE
  Mode		dd	?	; File_Map = Creation mode:
  				;   CREATE_ALWAYS or OPEN_EXISTING
				; File_UnMap = Delete flag
				;   TRUE to delete, FALSE to leave
  Attr		dd	?	; File attributes
  hFile		dd	?	; handle, file
  hMFile	dd	?	; handle, mapped file
  pMFile	dd	?	; pointer, mapped file in memory
MMF		ends

; File attributes.

FAttrAll	equ	FILE_ATTRIBUTE_READONLY or FILE_ATTRIBUTE_SYSTEM or \
			FILE_ATTRIBUTE_HIDDEN or FILE_ATTRIBUTE_ARCHIVE




;============================================================================
